
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*-------------------------- Driver X25 NAMTEL -------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*--------------------- (C) 1991 Jean-Claude MICHOT --------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/


#include <stdio.h>
#include <io.h>
#include <dev.h>
#include <tiny.h>
#include <task.h>
#include <task_info.h>
#include <task_msgs.h>
#include <magic.h>

#include "x25.h"

/*----------------------------------------------------------------------------*/

struct stat
	{
	long	stat_boot;		/* heure/date de lancement du driver */

	long	stat_pkt_nbr;	/* Nombre de trame recu */
	long	stat_pkt_nbe;	/* Nombre de trame envoye */

	long	stat_pkt_debu;	/* Erreur de debut de paquet? */
	long	stat_pkt_inco;	/* Nombre d'incoherence dans les paquets de donnees */
	long	stat_pkt_errm;	/* Erreur de multiplexage */
	long	stat_pkt_satu;	/* Nombre d'indication indisponibilite/saturation */
	long	stat_pkt_indi;	/* Nombre d'indication d'indisponibilite */
	};

/*----------------------------------------------------------------------------*/

struct msg msg;				/* Structure generale des msg/trame */
struct stat stat;			/* Statistique sur le concentrateur X25 */

/*----------------------------------------------------------------------------*/

#define DEB_PAQUET		0x02		/* Debut de paquet du concentrateur X25 */
#define FIN_PAQUET		0x03		/* Fin   de paquet du concentrateur X25 */

#define MAX_QUEUE_IN	64

#define FAUX 0
#define VRAI 1

/*----------------------------------------------------------------------------*/

static char buffer[300];

FILE		*fi,			/* Input */
			*fo,			/* Output */
			*fe;			/* Error */

char		device[20];		/* Device du concentrateur X25 */

int			debug;			/* Debug On/Off ? */

int			ref_xdev_pid;	/* Reference PID Device Qnx */
int			ref_xdev_port;	/* Reference Port Device Qnx */

int			pkt_indisponible,				/* Concentrateur indiponible */
			pkt_cv_saturer[ MAX_NB_CV+1 ];	/* Saturation d'un CV */

struct		msg	*qi[ MAX_QUEUE_IN+1 ];		/* queue in */
int			pos_queue;

/*----------------------------------------------------------------------------*/

/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * main( int argc, char *argv[] );                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 *                                                                         *
\***************************************************************************/

main( argc, argv )
int argc;
char *argv[];
{
	int		tid, i, r, baud, replysize;


	memset( &pkt_cv_saturer[0], '\0', sizeof( pkt_cv_saturer ) );

	pkt_indisponible = FAUX;
	baud             = 19200;
	debug            = 0;
	fe               = stderr;
	pos_queue        = 0;
	strcpy( device, "$mdm" );


	if ( !argc )
		error( 	"%s%s%s%s%s%s",
				"XADMIN, Par Jean-Claude MICHOT\n",
				"Version: (",__DATE__,")\n",
				"util: xadmin [options] &\n",
				"opts: +19200 +9600 +Debug +stat d=$device\n"
			);

	while(argc > 1)
	{
	char *p;
	p = argv[--argc];
	switch((*p << 8) | *(p + 1))
		{
		case '-9':
		case '+9':		baud=9600;						break;
		case '-1':
		case '+1':		baud=19200;						break;
		case '-D':
		case '+D':		debug=1;						break;
		case '-s':
		case '+s':		statistique();					break;
		case '-a':
		case 'a=':		fe = fopen( p+2, "a" );			break;
		case '-d':
		case 'd=':		strcpy( device, p+2 );			break;
						fi = fopen( device, "r" );
						if ( !fi ) 
							error("Impossible d'ouvrir la device\n");
						fclose ( fi );

						break;

		default:		error("Taper '%s' ? pour avoir la syntaxe\n", argv[0] );

						break;
		}
	}


	if ((Cmd_flags & RUN_BACKGROUND)==0)
		error( "Le Programme doit tourner en arriere plan.\n");

	if ( name_attach( "XADMIN", task_info(0,13) ) ==0 )
		error( "Impossible d'enregistrer le nom \"XADMIN\".\n");

	if ( set_priority( 9 ) != 0 )
		error( "Le programme doit etre (+privilegied)\n" );

	for( i=0; i<MAX_QUEUE_IN; i++ )
		{
		qi[ i ] = calloc( 1, sizeof( struct msg ) );
		if ( qi[i]==NULL )
			error( "No memory for input queue buffer\n" );
		}


	/* init de la device X25 */
	xinit_dev( device, baud );


	/* init des stats */
	memset( &stat, '\0', sizeof(stat) );
	time( &stat.stat_boot );


	while( 1 )
	{

	/* Reception d'un message, Si qq1 desire me dire qqchose? */
	tid = creceive( 0, &msg, &msg, sizeof(msg), sizeof(msg) );

	/* probablement a cause d'un signal lors du receive() */
	if ( tid==-1 )
		fprintf( fe, "XADMIN MAIN: creceive()==-1, signal?\n" );

	/* On a reelement recu qqchose de qq1 */
	if ( tid != 0 && tid != -1 )
		{
		replysize = 1;

		switch( msg.type )
			{

			case TYPE_STAT:
				memcpy( &msg.data, &stat, sizeof(stat) );
				replysize = sizeof(msg);
				break;

			case TYPE_GETDATA:
				getdata( msg.vc );
				replysize = sizeof(msg);
				break;

			case TYPE_PUTDATA:
				xsend_dev();
				break;

			case TYPE_ECHO_ON:
				echo_cv( msg.vc, VRAI );
				break;

			case TYPE_ECHO_OFF:
				echo_cv( msg.vc, FAUX );
				break;

			case TYPE_DISCONNECT:
				lib_cv( msg.vc,
						msg.data[0], /* cause */
						msg.data[1]  /* diagnostique */  );
				break;

			case TYPE_ADOPT_XDEVICE:
				ref_xdev_pid  = tid;
				ref_xdev_port = msg.data[0];
				break;

			case TYPE_UNADOPT_XDEVICE:
				ref_xdev_pid  = 0;
				ref_xdev_port = 0;
				break;

			default:
				break;
			}

		reply( tid, &msg, replysize );
		}


	/* Si on a des caracteres en attente de reception */
	if ( r=char_waiting( fi ) )
		xreceive_dev();

	/* Si rien en entree & rien en sortie, alors attend un peu */
	if ( r==0 )	nap( 1 );
	}
}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void xinit_dev(  char *devname, int baud );                             *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 * Init de la device physique de l'entree/sortie serie du                  *
 * concentrateur x25, et emission d'un RAS de l'ASM du concentrateur.      *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * - Nom de la device ou le concentrateur X25 se trouve.                   *
 * - vitesse de dialogue sur la ligne de ce concentrateur.                 *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * - Un message d'err s'il y a un probleme, dans le cas contraire          *
 *   la device est parametre afin d'etablir un dialogue avec le            *
 *   concentrateur.                                                        *
 *                                                                         *
\***************************************************************************/

void xinit_dev( devname, baud )
char *devname;
int baud;
{
	struct		dev_entry de;
	struct		stty_entry stty;
	char 		c;

	/* Ouverture en lecture puis ecriture de la device */
	fi = fopen( device, "r" );
	if ( !fi )
		error("XINIT_DEV: Impossible d'ouvrir la device '%s' (R)\n", device );
	fo = fopen( device, "w" );
	if ( !fo )
		error("XINIT_DEV: Impossible d'ouvrir la device '%s' (W)\n", device );

	if ( get_tty_parms( fi->_dev_no, &stty ) )
		error("XINIT_DEV: get_tty_parms() != 0\n");
	stty.stty_baud      = baud;
	stty.stty_parity    = 0;
	stty.stty_stop_bits = 1;
	stty.stty_data_bits = 8;
	if ( set_tty_parms( fi->_dev_no, &stty ) )
		error("XINIT_DEV: set_tty_parms() != 0\n");

	/* mise a zero des options de gestion de la device */
	set_option( fi, 0 );

	/* mise a zero des attribues de la device */
	get_attr( fi, &de );
	de.tty_BREAK = de.tty_ESCAPE = de.tty_RUBOUT = de.tty_CANCEL =
	de.tty_EOT   = de.tty_UP     = de.tty_RIGHT  = de.tty_DOWN   =
	de.tty_LEFT  = de.tty_INSERT = de.tty_DELETE = 0;
	set_attr( fi, &de );

	/* vide les buffer de sortie */
	fflush( fo );

	/* vide les buffer d'entree */
	flush_input( fi );
	while( char_waiting(fi) ) c=getc(fi);

	/* Trame de redemarrage de l'ASM */
	fprintf( fo, "%c%c%c%c%c%c",
			0x02, 0x3e, 0x64, 0x00, 0x00, 0x03 );
	fflush( fo );
}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void xsend_dev( void );                                                 *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 * Ecriture sur la device d'une trame que le concentrateur x25 sera en     *
 * mesure de comprendre et qui a ete constitue anterieurement.             *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * - Une structure globale "msg" qui comporte des informations coherentes. *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * - Envoi au concentrateur d'un message qu'il sera en mesure              *
 *   d'interpreter.                                                        *
 *                                                                         *
\***************************************************************************/

void xsend_dev()
{
	/* S'il y a eu des saturations au niveau de certain CV */
	while ( pkt_cv_saturer[ msg.vc ] == VRAI )
		xreceive_dev();

	/* S'il y a eu une indisponibilite generale */
	while ( pkt_indisponible == VRAI )
		xreceive_dev();

	/* Si la lng de ce qui est a ecrire > 0 */
	if ( msg.lng > 0 )
		{
		/* ecriture d'une trame de data qqconque */
		fput( &msg.data[0], msg.lng, fo );

		/* flush les buffer d'output pour envoi immediat */
		fflush( fo );

		/* statistique du nombre de paquet envoye a la carte */
		stat.stat_pkt_nbe++;
		}
}

/*----------------------------------------------------------------------------*/

mygetc( fp )
FILE *fp;
{
 char c;

 c = getc(fp);

 if ( debug )
	 fprintf( fe, "%02x,",c);

 return(c);

}

#define getc mygetc

/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void xreceive_dev( void );                                              *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
\***************************************************************************/

void xreceive_dev()
{
	register int r,i,lng;


	zero ( &buffer[0], 256 );

	fprintf( fe, "\n\nDEBUT DE TRAME:\n");

	stat.stat_pkt_nbr++;

	while( ( buffer[0]=getc(fi)) != DEB_PAQUET )
		{
		fprintf( fe, "Unknow: %02x '%c'\n", buffer[0], buffer[0] );

		stat.stat_pkt_debu++;	/* Erreur de debut de paquet? */

		if ( !char_waiting(fi) ) return;
		}

	buffer[1] = getc(fi);	/* Type de Message */
	buffer[2] = getc(fi);	/* Numero du Terminal */

	switch( (int) buffer[1] )
		{

		/* Trame d'information */
		case 0x30:
					fprintf( fe, "Trame INFO\n" );
					buffer[3] = getc(fi);	/* Lng Octet 1 */
					buffer[4] = getc(fi);	/* Lng Octet 2 */
					lng = (int) (buffer[3] & 0x7f) * 128 + (int) (buffer[4] & 0x7f);

					if ( lng )
						fget( buffer+5, lng+1, fi );

					receive_data( buffer[2], buffer+5, lng );

					break;

		/* Trame d'appel */
		case 0x31:
					fprintf( fe, "Trame APPEL\n" );

					buffer[3] = getc(fi);	/* Lng Octet 1 */
					buffer[4] = getc(fi);	/* Lng Octet 2 */
					lng = (int) (buffer[3] & 0x7f) * 128 + (int) (buffer[4] & 0x7f);

					if ( lng )
						fget( buffer+5, lng+1, fi );

					acceptation( buffer[2], buffer+5 );
					
					break;

		/* Trame de liberation */
		case 0x32:
					fprintf( fe, "Trame LIBE\n" );

					buffer[3] = getc(fi);	/* Lng Octet 1 */
					buffer[4] = getc(fi);	/* Lng Octet 2 */
					lng = (int) (buffer[3] & 0x7f) * 128 + (int) (buffer[4] & 0x7f);

					if ( lng )
						fget( buffer+5, lng+1, fi );

					if ( pkt_cv_saturer[ buffer[2] ] == VRAI )
						pkt_cv_saturer[ buffer[2] ] = FAUX;

					lib_cv( buffer[2], 0/*cause*/, 0/*diagnostique*/ );

					break;


		/* Trame incoherence dans donnees */
		case 0x3f:
					fprintf( fe, "Recu Trame INCO\n");
					fget( buffer+3, 4, fi );
					stat.stat_pkt_inco++;
					break;

		/* Trame d'indication d'erreur de procedure de multiplexage */
		case 0x34:
					fprintf( fe, "Recu Trame ERR Multiplexage\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					stat.stat_pkt_errm++;
					break;

		/* Trame d'indisponibilite / Saturation d'un CV */
		case 0x35:
					fprintf( fe, "Recu Trame INDISPONIBLE (saturation)\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );

					stat.stat_pkt_satu++;
					pkt_cv_saturer[ buffer[2] ] = VRAI;

					break;

		/* Trame de disponibilite / Fin de saturation */
		case 0x36:
					fprintf( fe, "Recu Trame DISPO\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );

					pkt_cv_saturer[ buffer[2] ] = FAUX;

					break;

		/* Trame Concentrateur Indisponible */
		case 0x39:
					fprintf( fe, "Trame Concentrateur KPUT (Attendre)\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );

					stat.stat_pkt_indi++;
					pkt_indisponible = VRAI;

					break;

		/* Trame Concentrateur Disponible */
		case 0x40:
					fprintf( fe, "Trame Concentrateur OK (Va-Bene)\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );

					pkt_indisponible = FAUX;

					break;

		/* Trame de donnees avec le bit Q a 1 */
		case 0x3a:
					fprintf( fe, "Trame DATA\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;


		/*----------------- ON NE DEVRAIS PAS RECEVOIR CECI -----------------*/

		/* Trame de suppression d'echo */
		case 0x37:
					fprintf( fe, "ATTENTION: Recu Trame (ECHO_OFF) ???\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;

		/* Trame de mise d'echo */
		case 0x38:
					fprintf( fe, "ATTENTION: Recu Trame (ECHO_ON) ???\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;

		/* Trame de Reset de du concentrateur */
		case 0x3e:
					fprintf( fe, "ATTENTION: Recu Trame 0x3e (RESET) ???\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;

		/* Trame Communication etablie */
		case 0x33:
					fprintf( fe, "ATTENTION: Recu Trame 0x33 (OKCOM) ???\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;
		defaut:
					fprintf( fe, "\nWARNING !!!! Unknow message\n");
					while( (buffer[3]=getc(fi)) != FIN_PAQUET );
					break;
		}

}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void statistique( void );                                               *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
\***************************************************************************/

void statistique()
{
	unsigned pid_xadmin;
	struct stat *sr;

	if( ( pid_xadmin=name_locate( "XADMIN", My_nid, sizeof(msg) ))==0)
		if((pid_xadmin=name_locate( "XADMIN", task_info(0,13), sizeof(msg) ))==0)
			if((pid_xadmin=name_locate( "XADMIN", 0, sizeof(msg) ))==0)
				error( "Impossible de trouver \"XADMIN\".\n\n");

	msg.type = TYPE_STAT;

	send( pid_xadmin, &msg, &msg, sizeof(msg), sizeof(msg) );

	sr = &msg.data[0];


	printf( "Statistique sur le programme XADMIN\n\n\n" );

	printf( "Data/Heure de lancement        : %s\n",
		cqtime( &sr->stat_boot ) );

	printf( "Nombre de paquet recu          : %ld\n", sr->stat_pkt_nbr );
	printf( "Nombre de paquet envoye        : %ld\n", sr->stat_pkt_nbe );
	printf( "Nombre de paquet d'incoherence : %ld\n", sr->stat_pkt_inco );

	exit(0);
}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void acceptation( unsigned char vc, char *data );                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
\***************************************************************************/

void acceptation( vc, data )
unsigned char vc;
char *data;
{
	if ( debug )
		fprintf( fe, "accecptation: cv(%02d) data(%s)\n", vc, data );

	/* Trame d'acception d'un CV */
	memset( &msg, '\0', sizeof( msg ) );
	msg.type	= -1;
	msg.vc		= vc;
	msg.lng		= 6;
	msg.data[0] = 0x02;
	msg.data[1] = 0x33;
	msg.data[2] = vc;
	msg.data[3] = 0x00;
	msg.data[4] = 0x00;
	msg.data[5] = 0x03;

	xsend_dev();
}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void lib_cv( unsigned char vc, cause, diagnostique );                   *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
\***************************************************************************/

void lib_cv( vc, cause, diagnostique )
unsigned char vc, cause, diagnostique;
{
	int i;

	if ( debug )
		fprintf( fe, "lib_vc: cv(%02d)\n", vc );

	/* Trame de liberation d'un CV */
	memset( &msg, '\0', sizeof( msg ) );
	msg.type	= -1;
	msg.vc		= vc;
	msg.lng		= 9;
	msg.data[0] = 0x02;
	msg.data[1] = 0x32;
	msg.data[2] = vc;
	msg.data[3] = 0x00;
	msg.data[4] = 0x03;
	msg.data[5] = cause; /* Cause */
	msg.data[6] = 0x2f;
	msg.data[7] = diagnostique; /* Diagnostique */
	msg.data[8] = 0x03;

	xsend_dev();

	/* liberation des tampons de ce CV dans la file d'attente */
	for( i=0; i<MAX_QUEUE_IN; i++ )
		if ( qi[ i ].vc == vc )
			memset( qi[i], '\0', sizeof(msg) );
}

/*----------------------------------------------------------------------------*/


/***************************************************************************\
 *                                                                         *
 * NOM ET PROTOTYPAGE DE LA FONCTION:                                      *
 *                                                                         *
 * void echo_cv( unsigned char vc, unsigned char on );                     *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * DESCRIPTION:                                                            *
 *                                                                         *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * INPUT:                                                                  *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
 *-------------------------------------------------------------------------*
 * OUTPUT:                                                                 *
 *                                                                         *
 * -                                                                       *
 *                                                                         *
\***************************************************************************/

void echo_cv( vc, on )
unsigned char vc, on;
{
	/* Trame de changement d'echo sur CV */
	memset( &msg, '\0', sizeof( msg ) );
	msg.type	= -1;
	msg.vc		= vc;
	msg.lng		= 6;
	msg.data[0] = 0x02;

	if ( on == VRAI )
		msg.data[1] = 0x38;
	else
	if ( on == FAUX )
		msg.data[1] = 0x37;
	else
		return;

	msg.data[2] = vc;
	msg.data[3] = 0x00;
	msg.data[4] = 0x00;
	msg.data[5] = 0x03;

	xsend_dev();
}

/*----------------------------------------------------------------------------*/

void dumper( buffer, lng, fd )
char *buffer;
int lng;
FILE *fd;
{
	register int	i;
	int				p;

	for ( p = 0; p < lng; p += 16 )
	{
	for ( i = 0; i < 16; i++ )
		fprintf( fd, "%02x ", (unsigned)(char)buffer[ i + p ] );

	fprintf ( fd, "| " );

	for ( i = 0; i < 16; i++ )
		fprintf( fd, "%c",
			( (unsigned)(char)buffer[ i + p ] > 31 ) ?
			( (unsigned)(char)buffer[ i + p ] < 0x80 ) ?
				buffer[ i + p ] : '.' : '.' );

	fprintf ( fd, "\n" );
	}

}

/*----------------------------------------------------------------------------*/
/*			Recuperation de donnee pour le gestionnaire de device			  */
/*----------------------------------------------------------------------------*/

getdata( vc )
unsigned char vc;
{
	register int i;

	for( i=pos_queue+1; i != pos_queue && qi[ i ].vc != vc; i++ )
		if ( i >= MAX_QUEUE_IN ) i = 0;

	if ( qi[i].vc == vc )
		{
		memcpy( &msg, qi[i], sizeof(msg) );
		memset( qi[i], '\0', sizeof( msg ) );
		return;
		}

	memset( &msg, '\0', sizeof( msg ) );
	return;
}

/*----------------------------------------------------------------------------*/
/*					Reception d'un paquet de donnees de la carte x25		  */
/*----------------------------------------------------------------------------*/

receive_data( vc, data, lng )
unsigned char vc;
char *data;
unsigned lng;
{
	char tmp[80];
	register int i;

	if ( debug )
		{
		fprintf( fe, "recu_de: cv(%02d) data(%s)\n", vc, data );
		dumper ( data, lng, fe );
		}

	qi[ pos_queue ].vc  = vc;
	qi[ pos_queue ].lng = lng;
	memcpy( qi[ pos_queue ].data, data, lng );

	if ( ++pos_queue >= MAX_QUEUE_IN ) pos_queue=0;
}

/*----------------------------------------------------------------------------*/
/*					Envoi d'un paquet de donnees a la carte x25				  */
/*----------------------------------------------------------------------------*/

send_data( vc, data, lng )
unsigned char vc;
char *data;
unsigned lng;
{
	/* Trame d'info server en maintenance */
	memset( &msg, '\0', sizeof( msg ) );
	msg.type	= -1;
	msg.vc		= vc;
	msg.lng		= 6+lng;
	msg.data[0] = 0x02;
	msg.data[1] = 0x30;
	msg.data[2] = vc;
	msg.data[3] = 0x00;
	msg.data[4] = lng;
	strcpy( msg.data+5, data );
	strcat( msg.data+5, "\003" );


	if ( debug )
		{
		fprintf( fe, "send_data(): lng=%d\n", lng );
		dumper ( &msg, lng+15, fe );
		}

	xsend_dev();
}

/*----------------------------------------------------------------------------*/
