
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*-------------------------- Driver X25 NAMTEL -------------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/*--------------------- (C) 1991 Jean-Claude MICHOT --------------------------*/
/*----------------------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/


/*----------------------------------------------------------------------------*/


#include <stdio.h>
#include <io.h>
#include <dev.h>
#include <lfsys.h>
#include <magic.h>
#include <task_msgs.h>
#include <ctype.h>
#include <exc.h>

#include "xdev.h"
#include "x25.h"


/*----------------------------------------------------------------------------*/


#define		printf		tprintf
#define		sprintf		tsprintf
#define		fprintf		tfprintf
#define		sscanf		tsscanf
#define		fscanf		tfscanf


#define		DEV_NAME	"pad"
#define		MAX_DEV		64


/*----------------------------------------------------------------------------*/



struct		attr_msg	*am;		/* Msg reply for GET_ATTR msg */
struct		stty_entry	*stty;		/* Msg reply for SET_OPTION msg */
			needdata;
char		buf			[ 100 + sizeof( struct msg_hdr ) ],
			input		[ 512+4 ],
			need,
			needdev		[ MAX_DEV ];
			data,
			adevname	[ 10 ];

int			deb_input,
			fin_input;

unsigned	tid,				/* Task ID of the sending task (cf: reveive) */
			cmd,				/* Type of message received */
			verbose,			/* Verbose ? False/True */
			debug,				/* Verbose ? False/True */
			devoptions[ MAX_DEV ];	/* Device Options */
int			i,
			devnum;				/* number of device to adopt */
int			devport;

char		xdevmap[ MAX_DEV ];

unsigned	pid_xadmin;			/* XAdmin PID */
struct		msg xmsg;			/* msg for Xadmin */


void		dumper();
int			device_write();
int			device_read();
void		echo_on();
void		echo_off();
void		fini();
void		USEID();
void		device();


/*----------------------------------------------------------------------------*/

main( argc, argv )
int		argc;
char	*argv[];
{

	verbose = debug = 0;
	devnum = 32;


	devport = 31;

	/* Options requested ? */

	if ( !argc || !(Cmd_flags & RUN_BACKGROUND) ) USEID();

	for ( i=argc-1; i>=1; i-- )
		{
		if ( strncmp( argv[i], "+v", 2 )==0 ) verbose=1;
		if ( strncmp( argv[i], "+d", 2 )==0 ) verbose=debug=1;
		if ( strncmp( argv[i], "-h", 2 )==0 ) USEID();
		if ( strncmp( argv[i], "-n", 2 )==0 ) devnum = atoi( argv[i]+2 );
		}

	if ( devnum < 1 ) devnum = 1;

	i = (task_info(0,15) & 0xff00) >> 8;
	if ( devnum > i )
		error("xdev: too many device. OS limit (%d)\n", i );


	gestion_xdevice();

	printf("\nxdev: starting %02d x25 pavi device\n", devnum );

	device();

	printf("xdev: startup ok x25 device running\n");

	exit(0);

}

/*----------------------------------------------------------------------------*/

void device()
{

	/*********************************************************************/

	need = data = 0;
	deb_input = fin_input = 0;

	for( i=0; i<MAX_DEV; i++ ) xdevmap    [i] = 0;
	for( i=0; i<MAX_DEV; i++ ) devoptions [i] = ECHO;


	for( i=0; i<devnum; i++ )
		{
		int tty;

		sprintf( adevname, "%s%02d", DEV_NAME, i );
		tty = adopt_device( -1, adevname );

		/* To translate TTY number to X25 link number */
		xdevmap[ tty ] = i;

		if ( tty == -1 )
			error("xdev: err (adopt_device)\n");

		}

	/*********************************************************************/

	/* Set the priority of device administrator */

	set_priority( 3 );


	/* Catch breaks and do shutdown when one ocurrs */

	exc_handler( EXC_BREAK, 0, &fini );


	/* Become Admin & Informed */

	if ( modify_flags( 1, ADMIN | INFORM ) )
		error("Must be a super user.\n");


	/*************************************************************************/

	for ( ;; )	
	{ /* begin for ever */

		memset( &buf[ 0 ], '\0', sizeof(buf)-1 );

		if ( debug )
			{
			fprintf( stderr, "\n\n" );
			}

		tid = receive( 0, &buf[0], sizeof( struct msg_hdr )+80 );

		if ( debug )
			{
			fprintf( stderr, "Receive Buffer:\n");
			dumper( buf, sizeof(buf), stderr );
			}

		cmd = buf.mtype;
		buf.mtype = OK;

		switch ( cmd )
		{ /* begin according to command typed */

			case OPEN:

				if ( verbose )
					fprintf( stderr, "*** OPEN REQUEST\n" );

				if ( buf.marg[ 0 ] & _APPEND )	/* Map append into write */
					buf.marg[ 0 ] |= _WRITE;

				if ( ( buf.marg[ 0 ] & ( _READ | _WRITE ) ) == ( _READ | _WRITE )
		   			||  ( buf.marg[ 0 ] & ( _EXECUTE | _DIRECTORY ) )
					||  ( buf.marg[ 1 ] & _FILE ) )
					{
					buf.mtype = 0xff;	/* Don't allow R/W or E opens */

					strcpy( buf.mbuf, "bad open mode for device\r\l" );

					reply( tid, &buf, sizeof( struct msg_hdr ) );

					break;
					}

				buf.mflags			= buf.marg[ 0 ] &
										( _READ | _WRITE | _UNBUF );
				buf.mdrive_number	= DEV_ASCII;
				buf.mindex			= 0;
				if ( buf.marg[0] & _WRITE )
					buf.mlen		= (buf.marg[0] & _UNBUF) ? 1 : 80;
				else
					buf.mlen		= 0;
				buf.mtid			= My_tid;

				if ( strncmp( buf.mbuf, "$tty", 4 ) != 0 )
					{
					buf.mtype = 0xff;	/* Don't allow $name opens */

					strcpy( buf.mbuf, "open $devname unauthorized\r\l" );

					reply( tid, &buf, sizeof( struct msg_hdr ) );

					break;
					}

				buf.mdev_no			= atoi( buf.mbuf+4 );

				buf.marg[ 0 ] = buf.marg[ 1 ] = 0;

				reply( tid, &buf, sizeof( struct msg_hdr ) );

				break;


			case EMPTY:

				if ( verbose )
					fprintf( stderr, "*** EMPTY REQUEST\n" );

				device_write(	map_tty_to_xdev(buf.mdev_no),
								&buf.mbuf[0],
								buf.mindex			);

				buf.mlen		= 80;
				buf.mindex		= 0;

				reply( tid, &buf, sizeof( struct msg_hdr ) );

				break;


			case FILL:

				if ( verbose )
					fprintf( stderr, "*** FILL REQUEST " );

				i = device_read(	map_tty_to_xdev(buf.mdev_no),
									&buf.mbuf[0],
									80				);

				if ( verbose )
					{
					fprintf( stderr, "device_read() = %d\n", i );
					if ( debug )
						dumper( buf.mbuf, 80, stderr );
					}

/*
				if ( i == 0 )
					{
					buf.mflags	|=	_EOF;
					buf.mtype	=	EOF;
					}
*/

				buf.mlen	=	i;
				buf.mindex	=	0;

				if ( i!=0 )
					reply( tid, &buf, sizeof( struct msg_hdr ) + buf.mlen );
				else
					{
					needdata = tid;
					need ++;
					}

				break;


			case CLOSE:

				if ( verbose )
					fprintf( stderr, "CLOSE\n" );

				/* close_xdevice( map_tty_to_xdev( buf.mdev_no ) ); */

				reply( tid, &buf, sizeof( struct msg_hdr ) );

				break;


			/*****************************************************************/

			case GET_ATTR:

				if ( verbose )
					fprintf( stderr, "GET_ATTR\n" );

				i = buf.marg[0]; /* tty to get attr */

				am = &buf[0];
				am->mtype = OK;
				am->devno = i;

				sprintf( adevname, "%s%02d", DEV_NAME, map_tty_to_xdev(i) );
				strcpy( am->tty_ent.dev_name, adevname );

				am->tty_ent.tty_ioport	=	0xCAFE;	/* Base Adr I/O */
				am->tty_ent.tty_type	=	DEV_ASCII;
				am->tty_ent.tty_options	=	devoptions[ i ];
			
				reply( tid, am, ATTR_SIZE + 2 ); 

				break;


			case SET_ATTR:

				if ( verbose )
					fprintf( stderr, "SET_ATTR\n" );

				i = buf.marg[0]; /* tty to set attr */

				am = &buf[0];
				devoptions[ i ] = am->tty_ent.tty_options;
				buf.mtype = OK;
				reply( tid, &buf[0], 1 );

				if ( devoptions[ i ] & ECHO )
					echo_on( map_tty_to_xdev( i ) );
				else
					echo_off( map_tty_to_xdev( i ) );

				break;


			case GET_OPTION:

				if ( verbose )
					fprintf( stderr, "GET_OPTION\n" );

				i = buf.marg[0]; /* tty to get options */
            	
                buf.marg[0] = devoptions[ i ];
				buf.marg[1] = i;
				buf.mtype = OK;
				reply( tid, &buf[0], 3 );

				break;


			case SET_OPTION:

				if ( verbose )
					fprintf( stderr, "SET_OPTION\n" );

				i = buf.marg[0]; /* tty to set options */

				buf.marg[0] = devoptions[ i ];
                devoptions[ i ] = buf.marg[1];
				buf.mtype = OK;
				reply( tid, &buf[0], 3 );

				break;


			case CHAR_WAITING:

				if ( verbose )
					fprintf( stderr, "CHAR_WAITING\n" );

				i = device_read( map_tty_to_xdev(buf.mdev_no), NULL, 0 );

				if ( i>255 ) i = 255;
				buf[0] = i;

				reply( tid, &buf[0], 1 );

				break;


			/*****************************************************************/


			case INHERIT_DEVICE:

				if ( verbose )
					fprintf( stderr, "INHERIT DEVICE\n" );

				buf.mtype = 0xff;
				reply( tid, &buf[0], 1 );

				break;
			

			case STTY_MESSAGE:

				stty = &buf[0];

				if ( verbose )
					fprintf( stderr, "STTY_MESSAGE\n" );

				buf.mtype = 0xff;

				switch( stty->stty_type )
				{
				case STTY_SET_PARAMETERS:
					if ( verbose )
						fprintf(stderr,"   STTY_SET_PARAMETERS\n");
					break;

				case STTY_QUERY_PARAMETERS:
					if ( verbose )
						fprintf(stderr,"   STTY_QUERY_PARAMETERS\n");
					stty->stty_baud			= 1200;
					stty->stty_parity		= 3;
					stty->stty_stop_bits	= 1;
					stty->stty_data_bits	= 7;
					buf.mtype = OK;
					break;

				case STTY_RESET_PARAMETERS:
					if ( verbose )
						fprintf(stderr,"   STTY_RESET_PARAMETERS\n");
					break;

				case STTY_SEND_BREAK:
					if ( verbose )
						fprintf(stderr,"   STTY_SEND_BREAK\n");
					break;

				case STTY_SET_OPTIONS:
					if ( verbose )
						fprintf(stderr,"   STTY_SET_OPTIONS\n");
					break;

				case STTY_GET_OPTIONS:
					if ( verbose )
						fprintf(stderr,"   STTY_GET_OPTIONS\n");
					break;

				case STTY_DROP_LINE:
					if ( verbose )
						fprintf(stderr,"   STTY_DROP_LINE\n");
					break;

				case STTY_CONTROL:
					if ( verbose )
						fprintf(stderr,"   STTY_CONTROL\n");
					break;

				case STTY_GET_TIME:
					if ( verbose )
						fprintf(stderr,"   STTY_GET_TIME\n");
					break;

				case STTY_SET_UOPTIONS:
					if ( verbose )
						fprintf(stderr,"   STTY_SET_UOPTIONS\n");
					break;

				case STTY_GET_UOPTIONS:
					if ( verbose )
						fprintf(stderr,"   STTY_GET_UOPTIONS\n");
					break;

				case STTY_XONXOFF:
					if ( verbose )
						fprintf(stderr,"   STTY_XONXOFF\n");
					break;

				default:
					if ( verbose )
						fprintf(stderr,"   WARNING: STTY UNKNOW\n");
					break;
				}

				reply( tid, &buf, sizeof( struct stty_entry )  );

				break;


			case MULTI_CHAR_WAITING:

				if ( verbose )
					fprintf( stderr, "MULTI_CHAR_WAITING\n" );

				buf.mtype = 0xff;
				reply( tid, &buf[0], 1 );

				break;


			case FLUSH_INPUT:

				if ( verbose )
					fprintf( stderr, "FLUSH_INPUT\n" );

				buf.mtype = 0xff;
				reply( tid, &buf[0], 1 );

				break;


			case NEW_OPTION:
			case DEV_TTY_TID:
			case DEV_TTY_ROOT_TID:
				break;

			case 0x7f:
				fill_inbuf( &buf[0] );
				break;

			case 0:
				relay( tid, 0 );
/*
				request_close_all();
*/
				break;

			default:

				if ( verbose )
					fprintf( stderr, "ANOTHER, UNSUPPORTED MSG REQUEST - %d\n", cmd );

				buf.mtype = 0xff;
				reply( tid, &buf[0], 1 );

				break;

		} /* end according to command typed */

	} /* end for ever */

	/*************************************************************************/
}

map_tty_to_xdev( tty )
int tty;
{
	return ( xdevmap[ tty ] );
}


/*----------------------------------------------------------------------------*/

void dumper( buffer, lng, fd )
char *buffer;
int lng;
FILE *fd;
{

	register int	i;
	int				p;

	for ( p = 0; p < lng; p += 16 )
	{

		for ( i = 0; i < 16; i++ )
			fprintf( fd, "%02x ", (unsigned)(char)buffer[ i + p ] );

		fprintf ( fd, "| " );

		for ( i = 0; i < 16; i++ )
			fprintf( fd, "%c",
				( (unsigned)(char)buffer[ i + p ] > 31 ) ?
				( (unsigned)(char)buffer[ i + p ] < 0x80 ) ?
					buffer[ i + p ] : '.' : '.' );

		fprintf ( fd, "\n" );

	}

}

/*----------------------------------------------------------------------------*/

int device_write( xdevice, buffer, lng )
char xdevice;
char *buffer;
int lng;
{

	memset( &xmsg, '\0', sizeof( xmsg ) );

	xmsg.type    = TYPE_PUTDATA;
	xmsg.vc	     = xdevice;
	xmsg.lng     = 6+lng;
	xmsg.data[0] = 0x02;
	xmsg.data[1] = 0x30;
	xmsg.data[2] = xdevice;
	xmsg.data[3] = 0x00;
	xmsg.data[4] = lng;
	strcpy( xmsg.data+5, buffer, lng );
	strcat( xmsg.data+5, "\003" );

	send( pid_xadmin, &xmsg, &xmsg, sizeof(xmsg), sizeof(xmsg) );

	return(0);
}

void echo_on( xdevice )
char xdevice;
{
	memset( &xmsg, '\0', sizeof( xmsg ) );

	xmsg.type    = TYPE_ECHO_ON;
	xmsg.vc	     = xdevice;

	send( pid_xadmin, &xmsg, &xmsg, sizeof(xmsg), sizeof(xmsg) );
}

void echo_off( xdevice )
char xdevice;
{
	memset( &xmsg, '\0', sizeof( xmsg ) );

	xmsg.type    = TYPE_ECHO_OFF;
	xmsg.vc	     = xdevice;

	send( pid_xadmin, &xmsg, &xmsg, sizeof(xmsg), sizeof(xmsg) );
}

int device_read( xdevice, buffer, lng )
char xdevice;
char *buffer;
int lng;
{
	return(0);
}

fill_inbuf( xbuf )
struct msg *xbuf;
{
	register int i=0;

	while ( xbuf->lng-- )
		{
		input[ fin_input++ ] = xbuf.data[ i++ ];
		if ( fin_input >= 512 ) fin_input=0;
		}
}

/*----------------------------------------------------------------------------*/

fini()
{
	if ( pid_xadmin )
		{
		memset( &xmsg, '\0', sizeof( xmsg ) );
		xmsg.type    = TYPE_UNADOPT_XDEVICE;

		send( pid_xadmin, &xmsg, &xmsg, sizeof(xmsg), sizeof(xmsg) );
		}

	exit(0);
}

/*----------------------------------------------------------------------------*/

USEID()
{
	error( "use: xdev [-nnumber_of_device -verbose -debug] &\n" );
}

/*----------------------------------------------------------------------------*/

gestion_xdevice()
{
	if( ( pid_xadmin=name_locate( "XADMIN", My_nid, sizeof(xmsg) ))==0)
		if((pid_xadmin=name_locate( "XADMIN", task_info(0,13), sizeof(xmsg) ))==0)
			if((pid_xadmin=name_locate( "XADMIN", 0, sizeof(xmsg) ))==0)
				{
				return;

				error( "Impossible de trouver \"XADMIN\".\n\n");
				}

	memset( &xmsg, '\0', sizeof(xmsg) );
	xmsg.type    = TYPE_ADOPT_XDEVICE;
	xmsg.data[0] = devport;

	send( pid_xadmin, &xmsg, &xmsg, sizeof(xmsg), sizeof(xmsg) );
}

/*----------------------------------------------------------------------------*/
