/*
 * TTY flags
 */

#define BUSY_WAIT		0x00
#define INT_DRIVEN		0x01
#define INT_FUNC		0x02
#define FUNC_CALL		0x03
#define TX_RDY			0x04
#define TTY_BUSY		0x08
#define BREAK_BIT		0x10
#define ECHO_XOFF		0x20
#define INS_FLAG		0x40
#define RE_ECHO			0x80
#define BREAK_PENDING	0x0100
#define ESCAPE_PENDING	0x0200
#define ECHO_XON		0x0400
#define INPUT_READY		0x0800
#define PRT_BUSY		0x1000
#define BREAK_ON		0x1000
#define HANGUP_PENDING	0x2000
#define FLOW_CONTROLLED	0x4000
#define PRT_READY		0x8000
#define DROP_ON			0x8000

/*
 * TTY EXTENDED FLAGS
 */

#define ECHO_PENDING	0x01
#define TRAIL_BACKSPACE	0x02
#define WINDOW_SWITCH	0x04
#define HOTKEY_PENDING	0x08
#define FUNCTION_KEY	0x10
#define PRECEDE_DEL		0x20
#define COMMERR_PENDING	0x40
#define INPUT_PENDING	0x80
#define DISABLE_WINDOW 	0x0800
#define MONO_CURSOR   	0x1000
#define DISABLE_REBOOT	0x2000
#define SPLIT_CONTROL	0x2000
#define POLL_PRINTER	0x4000
#define HANGUP			0x4000
#define DISALLOW_COLOUR	0x4000
#define LOCK_DEVICE		0x8000

#define PROMPT_MSG		0x0002

#define NUL		0x00
#define ETX		0x03
#define EOT		0x04
#define BS		0x08
#define HT		0x09
#define LF		0x0a
#define FF		0x0c
#define CR		0x0d
#define DC1		0x11
#define DC3		0x13
#define CAN		0x18
#define RS		0x1e
#define SP		0x20
#define DEL		0x7f

#define STATE	0
#define CHAR	1

#define HT_STATE	1
#define RS_STATE	8
#define DEL_STATE	5



struct edev_entry {
	char dev_name[6];
	unsigned tty_options;
	char tty_page;
	char tty_type;
	char tty_BREAK;
	char tty_ESCAPE;
	char tty_RUBOUT;
	char tty_CANCEL;
	char tty_EOT;
	char tty_UP;
	char tty_RIGHT;
	char tty_DOWN;
	char tty_LEFT;
	char tty_HOME;
	char tty_CLEAR;
	char tty_FUNCTION;
	char tty_INSERT;
	char tty_DELETE;
	unsigned tty_ioport;
	unsigned tty_ent_spare[2];
	char tty_hot_key;
	char tty_bs_count;
	unsigned tty_eflags;
	unsigned tty_flags;
	char tty_row;
	char tty_col;
	char tti_echo;
	char tti_ecnt;
	char tti_eptr;
	char tti_in;
	char tti_out;
	char tti_bol;
	char tti_eol;
	char tti_col;
	char tto_index;
	char tto_length;
	char tty_state;
	char tty_dev_no;
	char *tti_bufp;
	char *tto_bufp;
	unsigned dev_baud;			/* also printer timeout */
	char dev_parity;
	char dev_stop_bits;
	char dev_data_bits;
	char tti_echar;
	unsigned tto_dr;			/* also tto_function */
	unsigned char modem_status;
	char dev_entry_spare2;
	} ;

struct window_dev_entry {
	char window_dev_entry_filler[54];
	unsigned window_buf_seg;
	unsigned window_ctrl_block;
	char window_dev_entry_filler2[6];
	} ;

struct estate_entry {
	char state_cursor_x;
	char state_cursor_y;
	char state_attribute;
	char state_background;
	char state_esc_flag;
	char state_xpos;
	char state_cursor_type;
	char state_attr;
	char state_colour;
	char state_fill_colour;
	char state_screen_width;
	char state_screen_height;
	char state_attr_save;
	char state_colour_save;
	char state_fill_colour_save;
	char state_graph_mode;
	};

#define ATTR_SIZE 26

struct attr_msg {
	char mtype;
	char devno;
	struct dev_entry tty_ent;
	} ;

struct prompt_msg {
	char mtype;
	char new_disk;
	char on_tty;
	} ;

struct window_msg {
	char mtype;
	char wdw_devno;
	unsigned wdw_segment;
	char wdw_name[6];
	} ;


#define AWAIT_FILL	0x01
#define EOF_PENDING	0x02
#define INHERITED	0x04
#define TIMING		0x08
#define TIMEOUT		0x10

struct opt_entry {
	struct opt_entry *optlink;
	unsigned opt_dev_no;
	unsigned opt_save;
	} ;

struct open_entry {
	struct open_entry *oplink;
	unsigned open_tid;
	unsigned oflags;
	struct msg_hdr ohdr;
	char open_entry_filler; /* to make even */
	} ;

#define open_timeout ohdr.marg[0]
#define open_tlink ohdr.mlen

struct inherit_entry {
	struct open_entry *oplink;
	unsigned open_tid;
	unsigned oflags;
	} ;

struct output_buffer {
	struct output_buffer *olink;
	struct msg_hdr obuf;
	char ob_buf[80];
	char output_buffer_filler; /* to make even */
	} ;

struct abort_msg {
	char mtype;
	char my_tty;
	unsigned exc_bits;
	unsigned exc_addr;
	unsigned exc_seg;
	} ;

struct adopt_device_msg {
	char mtype;
	char mdevno;
	char mdev_name[6];
	} ;

struct query_device_msg {
	char mtype;
	char mdevno;
	char mdev_name[6];
	unsigned mdev_tid;
	} ;

struct new_device_entry {
	char new_dev_name[6];
	unsigned new_dev_tid;
	} ;

struct nacc_msg {
	char mtype;
	char mdevno;
	char mnacc;
	} ;

struct insert_char_msg {
	char mtype;
	char mdevno;
	unsigned minsert;
	} ;


#define STTY_SET_PARAMETERS		1
#define STTY_QUERY_PARAMETERS	2
#define STTY_RESET_PARAMETERS	3
#define STTY_SEND_BREAK			4
#define STTY_SET_OPTIONS		5
#define STTY_GET_OPTIONS		6
#define STTY_DROP_LINE			7
#define STTY_CONTROL			8
#define STTY_GET_TIME			9
#define STTY_SET_UOPTIONS		10
#define STTY_GET_UOPTIONS		11
#define STTY_XONXOFF			12

struct stty_entry {	/* message format */
	char 		mtype;			/* STTY_MESSAGE */
	char		stty_devno;
	char		stty_type;
	unsigned	stty_baud;		/* 0 .. 64K */
	char		stty_parity;	/* 0 - NONE, 1 - ODD, 3 - EVEN , 5 - MARK, 7 - SPACE */
	char		stty_stop_bits;	/* (1 .. 2) * 2 */
	char		stty_data_bits;	/* 5 .. 8 */
	} ;

struct stty_ctrl_msg {	/* message format */
	char 		mtype;			/* STTY_MESSAGE */
	char		stty_devno;
	char		stty_type;
	unsigned	stty_control;
	unsigned	stty_mask;
	} ;

struct multi_char_waiting_msg {
	char mtype;
	unsigned wndevs;
	struct wdata {
		unsigned char wdev_num;
		unsigned wnchars;
		} wdata[32];
	} ;

struct tty_time_msg {
	char 		mtype;
	long		lti_time;
	long		cur_time;
	} ;

struct dev_info_entry {
	unsigned info_num_devices;
	unsigned info_size_dev_entry;
	unsigned info_addr_dev_table;
	unsigned info_max_windows;
	unsigned info_size_screen_entry;
	unsigned info_addr_state_block;
	unsigned info_num_windows;
	unsigned info_current_window;
	unsigned info_current_dev_entry;
	unsigned info_addr_kbcodes;
	unsigned info_addr_compose_table;
	unsigned info_flag;
	unsigned info_addr_kbqueue;
	unsigned info_spare[10];
	} ;
